(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    (global = global || self, global.HighlightWords = factory());
}(this, function () { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    // https://stackoverflow.com/questions/26501688/a-typescript-guid-class
    function uuidv4() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            var r = (Math.random() * 16) | 0; // tslint:disable-line no-bitwise
            return (c == 'x' ? r : (r & 0x3) | 0x8).toString(16); // tslint:disable-line no-bitwise
        });
    }

    // We need escape certain characters before creating the RegExp
    // https://github.com/sindresorhus/escape-string-regexp
    var escapeRegexp = function (term) {
        return term.replace(/[|\\{}()[\]^$+*?.-]/g, function (char) { return "\\" + char; });
    };
    var regexpQuery = function (_a) {
        var terms = _a.terms, _b = _a.matchExactly, matchExactly = _b === void 0 ? false : _b;
        if (typeof terms !== 'string') {
            throw new TypeError('Expected a string');
        }
        var escapedTerms = escapeRegexp(terms);
        return "(" + (matchExactly ? escapedTerms : escapedTerms.split(' ').join('|')) + ")";
    };

    /**
     * This provides context around a chunk's text, based on the next and previous chunks.
     * e.g. If we have the string "The quick brown fox jumped over the lazy dog",
     * and the search term "fox jumped", with a padding of 2, we want to have the end result be:
     * "... quick brown fox jumped over the ..."
     * The search term, "fox jumped" is left as is, and the left and right chunks, instead of having
     * the text in full, will be clipped.
     */
    function clip(_a) {
        var curr = _a.curr, next = _a.next, prev = _a.prev, _b = _a.clipBy, clipBy = _b === void 0 ? 3 : _b;
        var words = curr.text.split(' ');
        var len = words.length;
        var ellipsis = len > clipBy ? '...' : '';
        // first we want to leave matches alone
        if (curr.match || clipBy >= len) {
            return curr.text;
        }
        else if (next && next.match) {
            // if we have a next chunk and it's a match
            return [ellipsis].concat(words.slice(-clipBy)).join(' ');
        }
        else if (prev && prev.match) {
            // if we have a previous chunk and it's a match
            return words.slice(0, clipBy).concat([ellipsis]).join(' ');
        }
        // on the off chance that there is no next and no previous chunk
        // we should really never get here
        return curr.text;
    }

    var hasLength = function (str) { return str.length > 0; };
    /**
     * Split a text into chunks denoting which are a match and which are not based on a user search term.
     * @param text          String  The text to split.
     * @param query         String  The query to split by. This can contain multiple words.
     * @param clipBy        Number  Clip the non-matches by a certain number of words to provide context around the matches.
     * @param matchExactly  Boolean If we have multiple words in the query, we will match any of the words if exact is false. For example, searching for "brown fox" in "the brown cute fox" will yield both "brown" and "fox" as matches. While if exact is true, the same search will return no results.
     */
    var highlightWords = function (_a) {
        var text = _a.text, query = _a.query, clipBy = _a.clipBy, _b = _a.matchExactly, matchExactly = _b === void 0 ? false : _b;
        if (query === '') {
            return [
                {
                    key: uuidv4(),
                    text: text,
                    match: false
                }
            ];
        }
        var searchRegexp = new RegExp(regexpQuery({ terms: query, matchExactly: matchExactly }), 'ig');
        return text
            .split(searchRegexp) // split the entire thing into an array of matches and non-matches
            .filter(hasLength) //filter any matches that have the text with length of 0
            .map(function (str) { return ({
            // compose the object for a match
            key: uuidv4(),
            text: str,
            match: matchExactly
                ? str.toLowerCase() === query.toLowerCase()
                : searchRegexp.test(str)
        }); })
            .map(function (chunk, index, chunks) { return (__assign({}, chunk, (clipBy && {
            // we only overwrite the text if there is a clip
            text: clip(__assign({ curr: chunk }, (index < chunks.length - 1 && { next: chunks[index + 1] }), (index > 0 && { prev: chunks[index - 1] }), { // if this wasn't the first chunk, set the previous chunk
                clipBy: clipBy }))
        }))); });
    };

    return highlightWords;

}));
